# Add sidebar to app

# If the ui.R, server.R and .html files of a shinySinglePage app is in "dir", then
# the \shinyTab based app in the current directory will have that app added as a sidebar.

# dropLink is initial text of a \href{}{} link that is converted to plain text (presumable)
addSidebar = function(dropLink=NULL, dir="sidebar") {
  # Get sidebar info
  info = file.info(dir)
  if (is.na(info$isdir)) stop(dir, " is not found in ", getwd())
  if (!info$isdir) stop(dir, " is not a directory")
  sbui = try(readLines(file.path(dir, "ui.R")))
  if (is(sbui, "try-error")) stop("'ui.R' not found in ", dir)
  sbserver = try(readLines(file.path(dir, "server.R")))
  if (is(sbserver, "try-error")) stop("'server.R' not found in ", dir)
  
  # get sidebar server code
  s1 = grep("^[[:space:]]*shinyServer[[:space:]]*[(].*[{]", sbserver)
  s2 = grep("^[[:space:]]*[}][[:space:]]*[)]", sbserver)
  if (length(s1)!=1 || length(s2)<1) stop("bad 'server.R'")
  s2 = s2[length(s2)]
  sbServerCode = if (s2>s1+1) sbserver[(s1+1):(s2-1)] else NULL
  
  # get sidebar ui code
  u0 = grep("^[[:space:]]*fluidPage[(]", sbui)
  u1 = grep("^[[:space:]]*singleton[(]tags[$]style", sbui)
  u2 = grep("^[[:space:]]*[)]", sbui)
  if (length(u0)!=1 || length(u1)<1 || length(u2)<1) stop("bad 'ui.R'")
  u1 = u1[length(u1)]
  u2 = u2[length(u2)]
  if (u2-u1 == 1) stop("no actual ui code")
  sbUiCode = sbui[(u1+1):(u2-1)]
  
  # get html files, rename to SB*, fix UI code, and put SB* copies of html file into main directory
  htmlLocs = grep('includeHTML[(]"[0-9]+-[0-9]+[.]html"[)],', sbUiCode)
  if (length(htmlLocs) > 0) {
    htmlFiles = regmatches(sbUiCode[htmlLocs], regexpr("[0-9]+-[0-9]+[.]html", sbUiCode[htmlLocs]))
    newHtmlFiles = paste0("SB", htmlFiles)
    for (i in 1:length(htmlFiles)) {
      sbUiCode[htmlLocs[i]] = sub(htmlFiles[i], newHtmlFiles[i], sbUiCode[htmlLocs[i]])
      file.copy(from = file.path(dir, htmlFiles[i]), to = newHtmlFiles[i], overwrite=TRUE)
    }
  }

  # Fix "drop" link
  if (!is.null(dropLink)) {
    LH = length(newHtmlFiles)
    if (LH==0) stop("dropLink present, but no html files are in sidebar UI")
    notFound = TRUE
    for (i in 1:LH) {
      txt = try(readLines(newHtmlFiles[LH-i+1]))
      if (is(txt, "try-error")) stop("cannot read ", newHtmlFiles[LH-i+1])
      gLine = intersect(grep(dropLink, txt, fixed=TRUE),
                        grep("href", txt, fixed=TRUE))
      if (length(gLine)==0) next
      if (length(gLine)>1) warning("only first match to ", dropLink, " changed")
      notFound = FALSE
      gLine = gLine[1]
      txt[gLine] = sub(paste0("(<a href='[^<]+?' target='_self'>)(", dropLink, ".*?)(</a>)"), "\\2", txt[gLine])
      write(txt, newHtmlFiles[LH-i+1])
    } # end for each html file in sidebar UI
    if (notFound) warning(dropLink, " not found in sidebar html")
  }
  
  
  # insert sidebar server code into main server code
  if (!is.null(sbServerCode)) {
    serverCode = try(readLines("server.R"))
    if (is(serverCode, "try-error")) stop("'server.R' not found")
    s1 = grep("^[[:space:]]*shinyServer[(].*[{]", sbserver)
    if (length(s1) != 1) stop("bad server code")
    serverCode = c(serverCode[1:s1], "", "  # added sidebar code", sbServerCode, 
                   "", "  # main server code", serverCode[s1:length(serverCode)])
    write(serverCode, "server.R")
  }
  
  # insert sidebar ui code into main ui code
  uiCode = try(readLines("ui.R"))
  if (is(uiCode, "try-error")) stop("'ui.R' not found")
  s0 = grep("^[[:space:]]*windowTitle[[:space:]]*[=]", uiCode)
  s1 = grep("^[[:space:]]*(navbar|fluid)Page[[:space:]]*[(]", uiCode)
  isFluid = length(grep("fluidPage", uiCode[s1])>1)
  s2 = grep("^[[:space:]]*(title[[:space:]]*[=]|titlePanel[[:space:]]*[(])", uiCode)
  s3 = grep("^[[:space:]]*tabPanel[[:space:]]*[(]", uiCode)
  if (!isFluid) s4 = grep("^[[:space:]]*[)][[:space:]]*[,]", uiCode)
  if (length(s0)>1) stop("bad 'ui.R' (multiple 'windowTitle =')")
  if (length(s1)!=1) stop("bad 'ui.R' (no 'navbar/fluid Page')")
  if (length(s2)<1) stop("bad 'ui.R' (no title)")
  s2 = s2[1]
  if (length(s3)>1) {
    s3 = s3[1]
    if (s2>s3) stop("bad 'ui.R' (first title is before first tab)")
  }
  if (!isFluid && length(s4)<1) stop("bad 'ui.R' (no '),')")
  if (!isFluid) s4 = s4[length(s4)]
  if (isFluid) {
    temp = regmatches(uiCode[s2], regexpr(', "([^"]*?)"),', uiCode[s2]))
    titleLine = paste0('  title = "', substring(temp, 4, nchar(temp)-3), '",')
    s4 = grep("[[:space:]]*[)][[:space:]]*#", uiCode)
    if (length(s4)==0) stop("bad ui code (no final isolated ')')")
    s4 = s4[length(s4)]
  } else {
    titleLine= paste0(sub("windowTitle", "title", uiCode[s0]), ",")
  }
  if ((!isFluid && (s0<s4 || s4<s2)) || s2<s1) stop("misordered ui code")
  uiCode[s1] = sub("(navbar|fluid)Page", "bootstrapPage", uiCode[s1])
  temp = c(uiCode[1:(s2-1)], titleLine,
           "  sidebarLayout(", "    sidebarPanel(", paste0("    ", sbUiCode), "    ),",
           "", "    mainPanel(")
  if (!isFluid) {
    if (s3 > s2+1) temp = c(temp, uiCode[(s2+1):(s3-1)])
    temp = c(temp, "", "      tabsetPanel(type = 'tabs',", paste0("      ", uiCode[s3:(s4-1)]),
            "  ))))", uiCode[(s4+1):(s0-1)], uiCode[(s0+1):length(uiCode)])
  } else {
    temp = c(temp, "", paste0("    ", uiCode[(s2+1):(s4-1)]), "    )", "  )", ")")
  }
  write(temp, "ui.R")
  
  invisible(NULL)
}