# Parse one line shinyTex commands (e.g., \shinySlider).  Called by innerCode().
#
# Arguments:
#   text: text containing one or more commands (but only one per line, and one line per command)
#
# Value:
#   Shiny ui code for each command
#
processCommands = function(text, before, globalInfoEnv) {
  lineNumOffset = before + get("lineNumOffset", envir=globalInfoEnv)
  rtn = NULL
  n = length(text)
  for (line in 1:n) {
    temp = parseCommand(text[line], lineNumOffset+line-1)
    type = temp$type
    options = temp$options
    
    # process command
    if (type == "shinySlider") {
      rtn = c(rtn, makeCall(type, "sliderInput", options, 
                            required = c("inputId", "label", "min", "max", "value"),
                            requiredTypes = c("c","c","n","n","n"),
                            optional = c("step", "round", "ticks", "animate", "width", "sep", "pre", "post"),
                            optionalTypes = c("n","n","l","l","c","c","c","c"),
                            lineNumOffset=lineNumOffset, globalInfoEnv=globalInfoEnv))
      
    } else if (type == "shinyDropdown") {
      rtn = c(rtn, makeCall(type, "selectInput", options, 
                              required = c("inputId", "label", "choices"),
                              requiredTypes = c("c","c","cv"),
                              optional = c("selected", "multiple", "selectize", "width", "size"),
                              optionalTypes = c("c","l","l","c","n"),
                              lineNumOffset=lineNumOffset, globalInfoEnv=globalInfoEnv))
        
    } else if (type == "shinyNumericInput") {
      rtn = c(rtn, makeCall(type, "numericInput", options, 
                            required = c("inputId", "label", "value"),
                            requiredTypes = c("c","c","n"),
                            optional = c("min", "max", "step"),
                            optionalTypes = c("n","n","n"),
                            lineNumOffset=lineNumOffset, globalInfoEnv=globalInfoEnv))
      
    } else if (type == "shinyActionButton") {
      rtn = c(rtn, makeCall(type, "actionButton", options, 
                            required = c("inputId", "label"), requiredTypes = c("c","c"),
                            optional = c("icon", "textColor", "backgroundColor", "borderColor", "style"), 
                            optionalTypes = rep("c",5),
                            lineNumOffset=lineNumOffset, globalInfoEnv=globalInfoEnv))

    } else if (type == "shinyRadioButtons") {
      rtn = c(rtn, makeCall(type, "radioButtons", options, 
                            required = c("inputId", "label", "choices"), requiredTypes = c("c","c","cv"),
                            optional = c("selected","inline"), optionalTypes = c("c","l"),
                            lineNumOffset=lineNumOffset, globalInfoEnv=globalInfoEnv))
      
    } else if (type == "shinyCheckbox") {
      rtn = c(rtn, makeCall(type, "checkboxInput", options, 
                            required = c("inputId", "label", "value"), requiredTypes = c("c","c","l"),
                            optional = NULL, optionalTypes = NULL,
                            lineNumOffset=lineNumOffset, globalInfoEnv=globalInfoEnv))
      
    } else if (type == "shinyTextInput") {
      opt2 = options
      temp = match("ncols", names(opt2))
      if (!is.na(temp)) {
        opt2$width = paste0(opt2$ncols, "em")
        opt2$ncols = NULL
      }
      rtn = c(rtn, makeCall(type, "textInput", opt2, 
                            required = c("inputId", "label"), requiredTypes = c("c","c"),
                            optional = c("value", "width"), optionalTypes = c("c","c"),
                            lineNumOffset=lineNumOffset, globalInfoEnv=globalInfoEnv))
      
    } else if (type == "shinyTextInputEnter") {
      rtn = c(rtn, makeCall(type, "textInputEnter", options, 
                            required = c("inputId"), requiredTypes = "c",
                            optional = c("label", "value", "width"), optionalTypes = c("c","c","c"),
                            lineNumOffset=lineNumOffset, globalInfoEnv=globalInfoEnv))
      
    } else if (type == "shinyTextAreaInput") {
      rtn = c(rtn, makeCall(type, "textareaInput", options, 
                            required = "inputId", requiredTypes = "c",
                            optional = c("value", "nrows", "ncols"), optionalTypes = c("c","n","n"),
                            lineNumOffset=lineNumOffset, globalInfoEnv=globalInfoEnv))
      
    } else if (type == "shinyQuiz") {
      rtn = c(rtn, makeCall(type, "quiz", options, 
                            required = c("bank"), requiredTypes = "c",
                            optional = c("addNumbers","randomizeQuestions","randomizeAnswers", "max", "interElement"), 
                            optionalTypes = c("l","l","l","n","c"), 
                            lineNumOffset=lineNumOffset, globalInfoEnv=globalInfoEnv))
      
    } else if (type == "shinyImage") {
      options = parseOptions(type, options, 
                             required = c("filename"), requiredTypes = "c",
                             optional = c("width","height"), 
                             optionalTypes = c("n","n"), lineNumOffset=lineNumOffset)
      str = paste0('HTML("<img src=\'', options$filename, '\'',
                   ifelse(any(names(options)=="width"), paste0("width='", options$width, "' "), ""),
                   ifelse(any(names(options)=="height"), paste0("height='", options$height, "' "), ""),
                   '>"),')
      rtn = c(rtn, str)
      #
      # store filename for later check
      filenames = getGlobal("filenames", globalInfoEnv)
      if (is.null(filenames)) {
        filenames = data.frame(filename=I(options$filename), type=I("image"))
      } else {
        filenames = rbind(filenames,
                          data.frame(filename=I(options$filename), type=I("image")))
      }
      assign("filenames", filenames, envir=globalInfoEnv)

      
    } else if (type == "shinyVideo") {
      options = parseOptions(type, options, 
                             required = c("filename"), requiredTypes = "c",
                             optional = c("width","height","type","autoplay"), 
                             optionalTypes = c("n","n","c","l"), lineNumOffset=lineNumOffset,
                             defaults = list(type="video/mp4"))
      str = c(paste0('HTML("<video ',
                     ifelse(any(names(options)=="width"), paste0("width='", options$width, "' "), ""),
                     ifelse(any(names(options)=="height"), paste0("height='", options$height, "' "), ""),
                     ifelse(any(names(options)=="autoplay") && options$autoplay, "autoplay ", ""),
                     ' controls>",'),
              paste0("\"  <source src='", options$filename, "' type='", options$type, "'>\","),
              "\"  <p>Your browser does not support &lt;video&gt;</p>\",",
              "\"</video>\"),")
      rtn = c(rtn, str)
      #
      # store filename for later check
      filenames = getGlobal("filenames", globalInfoEnv)
      if (is.null(filenames)) {
        filenames = data.frame(filename=I(options$filename), type=I("video"))
      } else {
        filenames = rbind(filenames,
                          data.frame(filename=I(options$filename), type=I("video")))
      }
      assign("filenames", filenames, envir=globalInfoEnv)
      
    } else if (type == "shinyAudio") {
      options = parseOptions(type, options, 
                             required = c("filename"), requiredTypes = "c",
                             optional = c("type","autoplay"), 
                             optionalTypes = c("c","l"), lineNumOffset=lineNumOffset,
                             defaults = list(type="audio/mp3"))
      str = c(paste0('HTML("<audio ',
                     ifelse(any(names(options)=="autoplay") && options$autoplay, "autoplay ", ""),
                     ' controls>",'),
              paste0("\"  <source src='", options$filename, "' type='", options$type, "'>\","),
              "\"  <p>Your browser does not support &lt;audio&gt;</p>\",",
              "\"</audio>\"),")
      rtn = c(rtn, str)
      #
      # store filename for later check
      filenames = getGlobal("filenames", globalInfoEnv)
      if (is.null(filenames)) {
        filenames = data.frame(filename=I(options$filename), type=I("audio"))
      } else {
        filenames = rbind(filenames,
                          data.frame(filename=I(options$filename), type=I("audio")))
      }
      assign("filenames", filenames, envir=globalInfoEnv)
      
    } else if (type == "shinyInsertCode") {
      options= parseOptions(type, options, required="name", requiredTypes="c",
                            optional = NULL, optionalTypes=NULL, 
                            lineNumOffset = lineNumOffset)
      code = get(options$name, envir = globalInfoEnv)
      browser()
    } else if (type == "shinyGotoTab") {
      options = parseOptions(type, options, 
                             required = c("name","text"), requiredTypes = c("c","c"),
                             optional = c("textColor","backgroundColor","borderColor","style"), 
                             optionalTypes = c("c","c","c","c"), lineNumOffset=lineNumOffset)
      #
      # Check for invalid tab
      allTabs = getGlobal("tabTitles", globalInfoEnv)
      if (is.null(allTabs)) stop("@", lineNumOffset, " attempt to use \\shinyGotoTab without any tabs")
      if (is.na(match(options$name, allTabs))) {
        stop("@", lineNumOffset, " \\shinyGotoTab names '", options$name, "' but the available tabs are ",
             paste(allTabs, collapse=", "))
      }
      #
      # store all gotoTab's and keep count for each tab (to allow unique ids if multiple jumps to any tab)
      gotoTabs = getGlobal("gotoTabs", globalInfoEnv)
      if (is.null(gotoTabs)) {
        count = 1
        gotoTabs = data.frame(tab=I(options$name), count=1)
      } else {
        pos = which(gotoTabs$tab == options$name)
        assure(length(pos)<=1, 302812)
        if (length(pos)==0) {
          count = 1
          gotoTabs = rbind(gotoTabs, data.frame(tab=I(options$name), count=1))
        } else {
          count = gotoTabs[pos, "count"] + 1
          gotoTabs[pos, "count"] = count
        }
      }
      assign("gotoTabs", gotoTabs, envir=globalInfoEnv)
      #
      # make the button
      style = makeStyle(options, lineNumOffset)
      str = paste0('actionButton("gotoTab', make.names(options$name), count, '", "', options$text, '"',
                   ifelse(is.null(style), '),',  paste0(',style="', style, '"),')))
      rtn = c(rtn, str)
         
    } else stop(type, " is not a valid \\shinySomeCommand command")
  } # end for each command
  return(rtn)
} # end processCommands()

